@@ -14,15 +14,28 @@ from sqlalchemy import select, delete # 导入 select, delete 用于异步操作
 import inspect
 from datetime import datetime
 from pro_craft.utils import extract_
-
+import asyncio
 import re
-
+from pydantic import BaseModel, ValidationError, field_validator
 from sqlalchemy import select, desc
 from json.decoder import JSONDecodeError
+from pro_craft.database import SyncMetadata
+from datetime import datetime, timedelta
+from datetime import datetime, timedelta
+from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine, async_sessionmaker
+from sqlalchemy import select, and_ # 引入 select 和 and_
+from sqlalchemy.orm import class_mapper # 用于检查对象是否是持久化的
+
 
 class IntellectRemoveFormatError(Exception):
     pass
 
+class IntellectRemoveError(Exception):
+    pass
+
+BATCH_SIZE = 100
+MIN_SUCCESS_RATE = 00.0 # 这里定义通过阈值, 高于该比例则通过
+
 
 def slog(s, target: str = "target",logger = None):
     COLOR_GREEN = "\033[92m"
@@ -59,6 +72,66 @@ def fix_broken_json_string(broken_json_str):
     return fixed_json_str
 
 
+# def get_last_sync_time(target_session) -> datetime:
+#     """从目标数据库获取上次同步时间"""
+#     metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="sync_metadata").first()
+#     if metadata_entry:
+#         return metadata_entry.last_sync_time
+#     return datetime(1970, 1, 1) # 默认一个很早的时间
+
+# def update_last_sync_time(target_session, new_sync_time: datetime):
+#     """更新目标数据库的上次同步时间"""
+#     metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="sync_metadata").first()
+#     if metadata_entry:
+#         metadata_entry.last_sync_time = new_sync_time
+#     else:
+#         # 如果不存在，则创建
+#         new_metadata = SyncMetadata(table_name="sync_metadata", last_sync_time=new_sync_time)
+#         target_session.add(new_metadata)
+#     target_session.commit()
+#     print(f"Updated last sync time to: {new_sync_time}")
+
+
+async def get_last_sync_time(target_session: AsyncSession) -> datetime:
+    """从目标数据库获取上次同步时间"""
+    # 修正点：使用 select() 和 execute()
+    result = await target_session.execute(
+        select(SyncMetadata).filter_by(table_name="ai_sync_metadata")
+    )
+    metadata_entry = result.scalar_one_or_none() # 获取单个对象或 None
+
+    if metadata_entry:
+        return metadata_entry.last_sync_time
+    return datetime(1970, 1, 1) # 默认一个很早的时间
+
+
+# from your_module import SyncMetadata # 假设 SyncMetadata 已导入
+# from sqlalchemy import select # 确保引入 select
+
+async def update_last_sync_time(target_session: AsyncSession, new_sync_time: datetime):
+    """更新目标数据库的上次同步时间"""
+    # 修正点：使用 select() 和 execute()
+    result = await target_session.execute(
+        select(SyncMetadata).filter_by(table_name="ai_sync_metadata")
+    )
+    metadata_entry = result.scalar_one_or_none()
+
+    if metadata_entry:
+        metadata_entry.last_sync_time = new_sync_time
+    else:
+        # 如果不存在，则创建
+        new_metadata = SyncMetadata(table_name="ai_sync_metadata", last_sync_time=new_sync_time)
+        target_session.add(new_metadata)
+    
+    # 异步提交事务
+    await target_session.commit() # TODO
+    print(f"Updated last sync time to: {new_sync_time}")
+
+
+
+
+
+
 class IntellectType(Enum):
     train = "train"
     inference = "inference"
@@ -143,6 +216,101 @@ class AsyncIntel():
 
         return specific_prompt
 
+    async def sync_prompt_data_to_database(self,database_url:str):
+        target_engine = create_async_engine(database_url, echo=False)
+        async with target_engine.begin() as conn:
+            await conn.run_sync(PromptBase.metadata.create_all)
+            
+        async with create_async_session(self.engine) as source_session:
+            async with create_async_session(target_engine) as target_session:
+            
+                last_sync_time = await get_last_sync_time(target_session)
+                print(f"Starting sync for sync_metadata from: {last_sync_time}")
+
+
+                processed_count = 0
+                #2 next_sync_watermark = last_sync_time
+                current_batch_max_updated_at = last_sync_time
+
+                while True:
+                    source_results = await source_session.execute(
+                        select(Prompt)
+                        .filter(Prompt.timestamp > last_sync_time)
+                        .order_by(Prompt.timestamp.asc(), Prompt.id.asc())
+                        .limit(BATCH_SIZE)
+                    )
+                    records_to_sync = source_results.scalars().all()
+                    if not records_to_sync:
+                        break # 没有更多记录了
+
+                    #2 max_timestamp_in_batch = datetime(1970, 1, 1) # 初始化为最早时间
+
+                    # 准备要插入或更新到目标数据库的数据
+                    for record in records_to_sync:
+                        # 查找目标数据库中是否存在该ID的记录
+                        # 这里的 `User` 模型会对应到 target_db.users
+                        target_prompt_result = await target_session.execute(
+                            select(Prompt).filter_by(id=record.id) # 假设 prompt_id 是唯一标识符
+                        )
+                        target_prompt = target_prompt_result.scalar_one_or_none()
+                        
+                        if target_prompt:
+                            # 如果存在，则更新
+                            target_prompt.prompt_id = record.prompt_id
+                            target_prompt.version = record.version
+                            target_prompt.timestamp = record.timestamp
+                            target_prompt.prompt = record.prompt
+                            target_prompt.use_case = record.use_case
+                            target_prompt.action_type = record.action_type
+                            target_prompt.demand = record.demand
+                            target_prompt.score = record.score
+                            target_prompt.is_deleted = record.is_deleted
+                        else:
+                            # 如果不存在，则添加新记录
+                            # 注意：这里需要创建一个新的User实例，而不是直接添加源数据库的record对象
+                            new_prompt = Prompt(
+                                prompt_id=record.prompt_id, 
+                                version=record.version,
+                                timestamp=record.timestamp,
+                                prompt = record.prompt,
+                                use_case = record.use_case,
+                                action_type = record.action_type,
+                                demand = record.demand,
+                                score = record.score,
+                                is_deleted = record.is_deleted
+                                )
+                            target_session.add(new_prompt)
+                        
+                        # 记录当前批次最大的 updated_at
+                        #2 
+                        # if record.timestamp > max_timestamp_in_batch:
+                        #     max_timestamp_in_batch = record.timestamp
+                        if record.timestamp > current_batch_max_updated_at:
+                            current_batch_max_updated_at = record.timestamp
+
+
+                    await target_session.commit() 
+                    processed_count += len(records_to_sync)
+                    print(f"Processed {len(records_to_sync)} records. Total processed: {processed_count}")
+
+                    #2 next_sync_watermark = max_timestamp_in_batch + timedelta(microseconds=1)
+                    last_sync_time = current_batch_max_updated_at + timedelta(microseconds=1) 
+
+                    
+                    if len(records_to_sync) < BATCH_SIZE: # 如果查询到的记录数小于批次大小，说明已经处理完所有符合条件的记录
+                        break
+
+                if processed_count > 0:
+                    # 最终更新last_sync_time到数据库，确保记录的是所有已处理记录中最新的一个
+                    await update_last_sync_time(target_session, current_batch_max_updated_at + timedelta(microseconds=1))
+
+                    #2 await update_last_sync_time(target_session, next_sync_watermark)
+
+                    await target_session.commit() # 确保最终的 metadata 更新也被提交
+                else:
+                    print("No new records to sync.")
+
+
     async def get_prompts_from_sql(self,
                              prompt_id: str,
                              version = None,
@@ -200,6 +368,21 @@ class AsyncIntel():
         session.add(prompt1)
         await session.commit() # 提交事务，将数据写入数据库
 
+    async def get_use_case_by_sql(self,
+                             target_prompt_id: str,
+                             session = None
+                            ):
+        """
+        从sql保存提示词
+        """
+        stmt = select(UseCase).filter(UseCase.is_deleted == 0,
+                                      UseCase.prompt_id == target_prompt_id)
+        
+        result = await session.execute(stmt)
+        # use_case = result.scalars().one_or_none()
+        use_case = result.scalars().all()
+        return use_case
+
     async def save_use_case_by_sql(self,
                              prompt_id: str,
                              use_case:str = "",
@@ -210,6 +393,8 @@ class AsyncIntel():
         """
         从sql保存提示词
         """
+        #TODO 存之前保证数据库中相同的prompt_id中没有重复的use_case
+
         use_case = UseCase(prompt_id=prompt_id, 
                         use_case = use_case,
                         output = output,
@@ -241,7 +426,8 @@ class AsyncIntel():
         chat_history = s_prompt or system_result
         await self.save_prompt_increment_version(prompt_id,
                                 new_prompt = chat_history,
-                                input_data = " summary ",
+                                use_case = "",
+                                score = 60,
                                 session = session)
         
     async def prompt_finetune_to_sql(
@@ -281,16 +467,18 @@ class AsyncIntel():
 {opinion}
 """
 
-        prompt, _ = await self.get_prompts_from_sql(prompt_id = prompt_id,version = version)
+        prompt_ = await self.get_prompts_from_sql(prompt_id = prompt_id,version = version,
+                                                    session=session)
         if demand:
             new_prompt = await self.llm.aproduct(
-                change_by_opinion_prompt.format(old_system_prompt=prompt, opinion=demand)
+                change_by_opinion_prompt.format(old_system_prompt=prompt_.prompt, opinion=demand)
             )
         else:
-            new_prompt = prompt
+            new_prompt = prompt_
         await self.save_prompt_increment_version(prompt_id = prompt_id,
                             new_prompt = new_prompt,
-                            input_data = " finetune ",
+                            use_case = "",
+                            score = 60,
                             session = session)
 
 
@@ -307,8 +495,8 @@ class AsyncIntel():
         async with create_async_session(self.engine) as session:
 
             latest_prompt = await self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
-
-            await self.save_prompt_increment_version(prompt_id=latest_prompt.prompt_id,
+            if latest_prompt:
+                await self.save_prompt_increment_version(prompt_id=latest_prompt.prompt_id,
                                     new_prompt = latest_prompt.prompt,
                                     use_case = latest_prompt.use_case,
                                     action_type=action_type,
@@ -316,8 +504,17 @@ class AsyncIntel():
                                     score=latest_prompt.score,
                                     session=session
                                     )
-            
-            return "success"
+                return "success"
+            else:
+                await self.save_prompt_increment_version(prompt_id=prompt_id,
+                                    new_prompt = demand,
+                                    use_case = "init",
+                                    action_type="inference",
+                                    demand=demand,
+                                    score=60,
+                                    session=session
+                                    )
+                return "init"
 
 
 
@@ -327,7 +524,7 @@ class AsyncIntel():
                     prompt_id: str,
                     version: str = None,
                     inference_save_case = True,
-                    push_patch = False,
+                    change_case = False,
                     ):
         if isinstance(input_data,dict):
             input_ = json.dumps(input_data,ensure_ascii=False)
@@ -337,27 +534,13 @@ class AsyncIntel():
         # 查数据库, 获取最新提示词对象
         async with create_async_session(self.engine) as session:
             result_obj = await self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
-
-
             if result_obj is None:
-                await self.save_prompt_increment_version(
-                    prompt_id = prompt_id,
-                    new_prompt = "做一些处理",
-                    use_case = input_,
-                    session = session
-                )
-                ai_result = await self.intellect_remove(input_data = input_data,
-                                       output_format = output_format,
-                                       prompt_id = prompt_id,
-                                       version = version,
-                                       inference_save_case = inference_save_case
-                                       )
-                return ai_result
+                raise IntellectRemoveError("不存在的prompt_id")
 
             prompt = result_obj.prompt
             if result_obj.action_type == "inference":
                 # 直接推理即可
-                ai_result = await self.llm.aproduct(prompt + output_format + "\n-----input----\n" +  input_)
+                ai_result = await self.llm.aproduct(prompt + output_format + "\nuser:" +  input_)
                 if inference_save_case:
                     await self.save_use_case_by_sql(prompt_id,
                                         use_case = input_,
@@ -376,21 +559,65 @@ class AsyncIntel():
                 demand = result_obj.demand
             
 
-                assert demand
-                # 注意, 这里的调整要求使用最初的那个输入, 最好一口气调整好
-                chat_history = prompt
-                if input_ == before_input: # 输入没变, 说明还是针对同一个输入进行讨论
-                    # input_prompt = chat_history + "\nuser:" + demand
-                    input_prompt = chat_history + "\nuser:" + demand + output_format 
-                else:
-                    # input_prompt = chat_history + "\nuser:" + demand + "\n-----input----\n" + input_
-                    input_prompt = chat_history + "\nuser:" + demand + output_format  + "\n-----input----\n" + input_
+                # assert demand
+                # # 注意, 这里的调整要求使用最初的那个输入, 最好一口气调整好
+                # chat_history = prompt
+                # if input_ == before_input: # 输入没变, 说明还是针对同一个输入进行讨论
+                #     # input_prompt = chat_history + "\nuser:" + demand
+                #     input_prompt = chat_history + "\nuser:" + demand + output_format 
+                # else:
+                #     # input_prompt = chat_history + "\nuser:" + demand + "\n-----input----\n" + input_
+                #     input_prompt = chat_history + "\nuser:" + demand + output_format  + "\n-----input----\n" + input_
             
-                ai_result = await self.llm.aproduct(input_prompt)
-                chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词
-                await self.save_prompt_increment_version(prompt_id, chat_history,
-                                        use_case = input_,
-                                        session = session)
+                # ai_result = await self.llm.aproduct(input_prompt)
+                # chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词
+                # await self.save_prompt_increment_version(prompt_id, chat_history,
+                #                         use_case = input_,
+                #                         score = 60,
+                #                         session = session)
+
+
+                # version 2
+
+                # if input_ == before_input:
+                #     new_prompt = prompt + "\nuser:" + demand
+                # else:
+                #     new_prompt = prompt + "\nuser:" + input_
+
+                # ai_result = await self.llm.aproduct(new_prompt + output_format)
+
+                # save_new_prompt = new_prompt + "\nassistant:\n" + ai_result
+
+
+                # await self.save_prompt_increment_version(
+                #     prompt_id, 
+                #     new_prompt=save_new_prompt,
+                #     use_case = input_,
+                #     action_type = "inference",
+                #     score = 60,
+                #     session = session)
+                
+                if before_input == "" or change_case is True:
+                    result_obj.use_case = input_
+                    await session.commit()
+                    # 查询上一条, 将before_input 更新位input_
+                    prompt += input_
+
+                # 使用更新后的数据进行后续步骤
+                new_prompt = prompt + "\nuser:" + demand
+
+                ai_result = await self.llm.aproduct(new_prompt + output_format)
+
+                save_new_prompt = new_prompt + "\nassistant:\n" + ai_result
+
+
+                await self.save_prompt_increment_version(
+                    prompt_id, 
+                    new_prompt=save_new_prompt,
+                    use_case = input_,
+                    action_type = "inference",
+                    score = 60,
+                    session = session)
     
             elif result_obj.action_type == "summary":
 
@@ -398,7 +625,7 @@ class AsyncIntel():
                             prompt = prompt,
                             session = session
                             )
-                ai_result = await self.llm.aproduct(prompt + output_format + "\n-----input----\n" +  input_)
+                ai_result = await self.llm.aproduct(prompt + output_format + "\nuser:" +  input_)
 
             elif result_obj.action_type == "finetune":
                 demand = result_obj.demand
@@ -408,20 +635,17 @@ class AsyncIntel():
                                             demand = demand,
                                             session = session
                                             )
-                ai_result = await self.llm.aproduct(prompt + output_format + "\n-----input----\n" +  input_)
+                ai_result = await self.llm.aproduct(prompt + output_format + "\nuser:" +  input_)
             elif result_obj.action_type == "patch":
-                
                 demand = result_obj.demand
                 assert demand
-
                 chat_history = prompt + demand
-                ai_result = await self.llm.aproduct(chat_history + output_format + "\n-----input----\n" +  input_)
-
-                if push_patch:
-                    self.save_prompt_increment_version(prompt_id, chat_history,
-                                            use_case = input_,
-                                            score = 60,
-                                            session = session)
+                ai_result = await self.llm.aproduct(chat_history + output_format + "\nuser:" +  input_)
+                self.save_prompt_increment_version(prompt_id, 
+                                                   chat_history,
+                                                    use_case = input_,
+                                                    score = 60,
+                                                    session = session)
 
             else:
                 raise
@@ -508,7 +732,9 @@ class AsyncIntel():
                 chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词
                 await self.save_prompt_increment_version(prompt_id, chat_history,
                                         use_case = input_,
+                                        score = 60,
                                         session = session)
+
     
             elif result_obj.action_type == "summary":
 
@@ -580,11 +806,10 @@ class AsyncIntel():
                     output_format=output_format,
                     prompt_id=prompt_id,
                     version=version,
-                    inference_save_case=inference_save_case
+                    inference_save_case=inference_save_case,
                 )
 
         try:
-
             json_str = extract_(ai_result,r'json')
             # json_str = fix_broken_json_string(json_str)
             ai_result = json.loads(json_str)
@@ -599,10 +824,47 @@ class AsyncIntel():
                 OutputFormat(**ai_result)
 
             except JSONDecodeError as e:
-                raise IntellectRemoveFormatError(f"prompt_id: {prompt_id} 在生成后做json解析时报错") from e
+                raise IntellectRemoveFormatError(f"prompt_id: {prompt_id} 生成的内容为无法被Json解析 {e}") from e
+        
+        except ValidationError as e:
+            err_info = e.errors()[0]
+            raise IntellectRemoveFormatError(f"{err_info["type"]}: 属性:{err_info['loc']}, 发生了如下错误: {err_info['msg']}, 格式校验失败, 当前输入为: {err_info['input']} 请检查") from e
+
+        except Exception as e:
+            raise Exception(f"Error {prompt_id} : {e}") from e
 
         return ai_result
     
+    async def intellect_remove_formats(self,
+                    input_datas: list[dict | str],
+                    OutputFormat: object,
+                    prompt_id: str,
+                    ExtraFormats: list[object] = [],
+                    version: str = None,
+                    inference_save_case = True,
+                    ):
+                
+        async with create_async_session(self.engine) as session:
+            prompt_result = await self.get_prompts_from_sql(prompt_id=prompt_id,
+                                                                   session=session)
+            if prompt_result is None:
+                raise IntellectRemoveError("不存在的prompt_id")
+        if prompt_result.action_type != "inference":
+            input_datas = input_datas[:1]
+        tasks = []
+        for input_data in input_datas:
+            tasks.append(
+                self.intellect_remove_format(
+                    input_data = input_data,
+                    prompt_id = prompt_id,
+                    OutputFormat = OutputFormat,
+                    ExtraFormats = ExtraFormats,
+                    version = version,
+                    inference_save_case = inference_save_case,
+                )
+            )
+        results = await asyncio.gather(*tasks, return_exceptions=False)
+        return results
     
     def intellect_remove_warp(self,prompt_id: str):
         def outer_packing(func):
@@ -631,3 +893,90 @@ class AsyncIntel():
                 return result
             return wrapper
         return outer_packing
+
+    async def intellect_remove_format_eval(self,
+                    OutputFormat: object,
+                    prompt_id: str,
+                    ExtraFormats: list[object] = [],
+                    version: str = None,
+                    ):
+        
+        async with create_async_session(self.engine) as session:
+            use_cases = await self.get_use_case_by_sql(target_prompt_id=prompt_id,session=session)
+            prompt_result = await self.get_prompts_from_sql(prompt_id=prompt_id,
+                                                                   session=session)
+            if prompt_result is None:
+                raise IntellectRemoveError("不存在的prompt_id")
+            if prompt_result.action_type != "inference":
+                raise IntellectRemoveError("请在inference模式下使用次类")
+            
+
+            total_assertions = len(use_cases)
+            result_cases = []
+
+            async def evals_func(use_case,prompt_id,OutputFormat,ExtraFormats,version):
+                try:
+                    # 这里将参数传入
+                    await self.intellect_remove_format(
+                        input_data = use_case.use_case,
+                        prompt_id = prompt_id,
+                        OutputFormat = OutputFormat,
+                        ExtraFormats = ExtraFormats,
+                        version = version,
+                        inference_save_case = False,
+                    )
+                    # TODO base_eval
+                    # TODO 人类评价 eval
+                    # TODO llm 评价 eval
+                    result_cases.append({"type":"Successful","case":use_case.use_case,"reply":f"pass"})
+                    use_case.output = "Successful"
+                except IntellectRemoveFormatError as e:
+                    result_cases.append({"type":"FAILED","case":use_case.use_case,"reply":f"{e}"})
+                    use_case.output = f"{"FAILED"}-{e}"
+                except Exception as e: # 捕获其他可能的错误
+                    result_cases.append({"type":"FAILED","case":use_case.use_case,"reply":f"Exp {e}"})
+                    use_case.output = f"{"FAILED"}-{e}"
+                    await session.commit()
+
+            tasks = []
+            for use_case in use_cases:
+                tasks.append(
+                    evals_func(
+                        use_case = use_case,
+                        prompt_id = prompt_id,
+                        OutputFormat = OutputFormat,
+                        ExtraFormats = ExtraFormats,
+                        version = version
+                    )
+                )
+            await asyncio.gather(*tasks, return_exceptions=False)
+
+
+            successful_assertions = 0
+            bad_case = []
+            for i in result_cases:
+                if i['type'] == "Successful":
+                    successful_assertions += 1
+                else:
+                    bad_case.append(i)
+
+            success_rate = (successful_assertions / total_assertions) * 100
+            print(f"\n--- Aggregated Results ---")
+            print(f"Total test cases: {total_assertions}")
+            print(f"Successful cases: {successful_assertions}")
+            print(f"Success Rate: {success_rate:.2f}%")
+
+            # if success_rate >= MIN_SUCCESS_RATE:
+            #     return "通过", json.dumps(result_cases,ensure_ascii=False)
+            # else:
+            #     return "未通过",json.dumps(result_cases,ensure_ascii=False)
+
+            print(bad_case)
+
+            
+            # return results
+
+
+
+
+# 整体测试d, 测试未通过d, 大模型调整再测试, 依旧不通过, 大模型裂变, 仍不通过, 互换人力
