@@ -22,6 +22,8 @@ BATCH_SIZE = 1000
 class IntellectRemoveFormatError(Exception):
     pass
 
+class IntellectRemoveError(Exception):
+    pass
 
 def slog(s, target: str = "target",logger = None):
     COLOR_GREEN = "\033[92m"
@@ -37,19 +39,19 @@ def slog(s, target: str = "target",logger = None):
 
 def get_last_sync_time(target_session) -> datetime:
     """从目标数据库获取上次同步时间"""
-    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="sync_metadata").first()
+    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="ai_sync_metadata").first()
     if metadata_entry:
         return metadata_entry.last_sync_time
     return datetime(1970, 1, 1) # 默认一个很早的时间
 
 def update_last_sync_time(target_session, new_sync_time: datetime):
     """更新目标数据库的上次同步时间"""
-    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="sync_metadata").first()
+    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="ai_sync_metadata").first()
     if metadata_entry:
         metadata_entry.last_sync_time = new_sync_time
     else:
         # 如果不存在，则创建
-        new_metadata = SyncMetadata(table_name="sync_metadata", last_sync_time=new_sync_time)
+        new_metadata = SyncMetadata(table_name="ai_sync_metadata", last_sync_time=new_sync_time)
         target_session.add(new_metadata)
     target_session.commit()
     print(f"Updated last sync time to: {new_sync_time}")
@@ -354,17 +356,28 @@ class Intel():
         # 查看是否已经存在
         with create_session(self.engine) as session:
             latest_prompt = self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
-
-            self.save_prompt_increment_version(prompt_id=latest_prompt.prompt_id,
-                                    new_prompt = latest_prompt.prompt,
-                                    use_case = latest_prompt.use_case,
-                                    action_type=action_type,
+               
+            if latest_prompt:
+                self.save_prompt_increment_version(prompt_id=latest_prompt.prompt_id,
+                                        new_prompt = latest_prompt.prompt,
+                                        use_case = latest_prompt.use_case,
+                                        action_type=action_type,
+                                        demand=demand,
+                                        score=latest_prompt.score,
+                                        session=session
+                                        )
+                
+                return "success"
+            else:
+                self.save_prompt_increment_version(prompt_id=prompt_id,
+                                    new_prompt = demand,
+                                    use_case = "init",
+                                    action_type="inference",
                                     demand=demand,
-                                    score=latest_prompt.score,
+                                    score=60,
                                     session=session
                                     )
-            
-            return "success"
+                return "init"
 
     def intellect_remove(self,
                     input_data: dict | str,
@@ -393,19 +406,7 @@ class Intel():
             result_obj = self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
 
             if result_obj is None:
-                self.save_prompt_increment_version(
-                    prompt_id = prompt_id,
-                    new_prompt = "做一些处理",
-                    use_case = input_,
-                    session = session
-                )
-                ai_result = self.intellect_remove(input_data = input_data,
-                                       output_format = output_format,
-                                       prompt_id = prompt_id,
-                                       version = version,
-                                       inference_save_case = inference_save_case
-                                       )
-                return ai_result
+                raise IntellectRemoveError("不存在的prompt_id")
             prompt = result_obj.prompt
             if result_obj.action_type == "inference":
                 # 直接推理即可
@@ -423,24 +424,35 @@ class Intel():
                 # 则训练推广
 
                 # 新版本 默人修改会 inference 状态
-                chat_history = prompt
+                prompt = result_obj.prompt
                 before_input = result_obj.use_case
                 demand = result_obj.demand
-            
 
-                assert demand
-                # 注意, 这里的调整要求使用最初的那个输入, 最好一口气调整好
-                chat_history = prompt
-                if input_ == before_input: # 输入没变, 说明还是针对同一个输入进行讨论
-                    # input_prompt = chat_history + "\nuser:" + demand
-                    input_prompt = chat_history + "\nuser:" + demand + output_format 
-                else:
-                    # input_prompt = chat_history + "\nuser:" + demand + "\n-----input----\n" + input_
-                    input_prompt = chat_history + "\nuser:" + demand + output_format  + "\n-----input----\n" + input_
+                # assert demand
+                # # 注意, 这里的调整要求使用最初的那个输入, 最好一口气调整好
+                
+                # if input_ == before_input: # 输入没变, 说明还是针对同一个输入进行讨论
+                #     # input_prompt = chat_history + "\nuser:" + demand
+                #     input_prompt = chat_history + "\nuser:" + demand + output_format 
+                # else:
+                #     # input_prompt = chat_history + "\nuser:" + demand + "\n-----input----\n" + input_
+                #     input_prompt = chat_history + "\nuser:" + demand + output_format  + "\n-----input----\n" + input_
             
-                ai_result = self.llm.product(input_prompt)
-                chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词
-                self.save_prompt_increment_version(prompt_id, chat_history,
+                # ai_result = self.llm.product(input_prompt)
+                # chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词
+
+                if input_ == before_input:
+                    new_prompt = prompt + "\nuser:" + demand
+                else:
+                    new_prompt = prompt + "\nuser:" + input_
+
+                ai_result = self.llm.product(new_prompt + output_format)
+
+                save_new_prompt = new_prompt + "\nassistant:\n" + ai_result
+
+
+                self.save_prompt_increment_version(prompt_id, 
+                                        new_prompt=save_new_prompt,
                                         use_case = input_,
                                         score = 60,
                                         session = session)
